@@ -38,10 +38,12 @@
 #include "flac_utils.h"
 #include "from_flac_library.h"
 #include "flac_metadata_utils.h"
+#include "silence_processors.h"
+#include "flac_silence.h"
 
 static void splt_flac_fr_read_header(splt_flac_frame_reader *fr,
     unsigned min_blocksize, unsigned max_blocksize, unsigned metadata_bits_per_sample, 
-    splt_flac_code *error)
+    splt_state *state, splt_code *error)
 {
   fr->crc8 = 0;
   fr->bytes_between_frame_number_and_crc8 = 0;
@@ -171,23 +173,27 @@ static void splt_flac_fr_read_header(splt_flac_frame_reader *fr,
   unsigned char crc8 = splt_flac_u_read_next_byte(fr, error);
   if (*error < 0) { return; }
 
-  if (crc8 != computed_crc8) { *error = SPLT_FLAC_ERR_BAD_CRC8; }
+  if (crc8 != computed_crc8)
+  {
+    splt_e_set_error_data(state, splt_t_get_filename_to_split(state));
+    *error = SPLT_ERROR_INVALID;
+  }
 }
 
 static void splt_flac_fr_read_constant_subframe(splt_flac_frame_reader *fr, unsigned bits_per_sample,
-    splt_flac_code *error)
+    splt_code *error)
 {
   splt_flac_u_read_up_to_total_bits(fr, bits_per_sample, error);
 }
 
 static void splt_flac_fr_read_verbatim_subframe(splt_flac_frame_reader *fr, unsigned bits_per_sample,
-    splt_flac_code *error)
+    splt_code *error)
 {
   splt_flac_u_read_up_to_total_bits(fr, bits_per_sample * fr->blocksize, error);
 }
 
 static void splt_flac_fr_read_rice_residual(splt_flac_frame_reader *fr, unsigned order,
-    splt_flac_code *error)
+    splt_code *error)
 {
   unsigned char rice_method = 4;
 
@@ -249,7 +255,7 @@ static void splt_flac_fr_read_rice_residual(splt_flac_frame_reader *fr, unsigned
 }
 
 static void splt_flac_fr_read_fixed_subframe(splt_flac_frame_reader *fr, unsigned order, 
-    unsigned bits_per_sample, splt_flac_code *error)
+    unsigned bits_per_sample, splt_code *error)
 {
   splt_flac_u_read_up_to_total_bits(fr, bits_per_sample * order, error);
   if (*error < 0) { return; }
@@ -258,7 +264,7 @@ static void splt_flac_fr_read_fixed_subframe(splt_flac_frame_reader *fr, unsigne
 }
 
 static void splt_flac_fr_read_lpc_subframe(splt_flac_frame_reader *fr, unsigned order, 
-    unsigned bits_per_sample, splt_flac_code *error)
+    unsigned bits_per_sample, splt_state *state, splt_code *error)
 {
   splt_flac_u_read_up_to_total_bits(fr, bits_per_sample * order, error);
   if (*error < 0) { return; }
@@ -268,9 +274,8 @@ static void splt_flac_fr_read_lpc_subframe(splt_flac_frame_reader *fr, unsigned
 
   if ((quantized_linear_predictor & 0x0f) == 0x0f)
   {
-    //TODO: error
-    fprintf(stderr, "error in lpc subframe quantized predictor\n");
-    fflush(stderr);
+    splt_e_set_error_data(state, splt_t_get_filename_to_split(state));
+    *error = SPLT_ERROR_INVALID;
   }
 
   //qlp_coeff_shift
@@ -285,7 +290,7 @@ static void splt_flac_fr_read_lpc_subframe(splt_flac_frame_reader *fr, unsigned
 }
 
 static void splt_flac_fr_read_subframe_content(splt_flac_frame_reader *fr, int sf_type, unsigned order,
-    unsigned bits_per_sample, splt_flac_code *error)
+    unsigned bits_per_sample, splt_state *state, splt_code *error)
 {
   switch (sf_type)
   {
@@ -299,15 +304,16 @@ static void splt_flac_fr_read_subframe_content(splt_flac_frame_reader *fr, int s
       splt_flac_fr_read_fixed_subframe(fr, order, bits_per_sample, error);
       return;
     case SPLT_FLAC_SUBFRAME_LPC:
-      splt_flac_fr_read_lpc_subframe(fr, order, bits_per_sample, error);
+      splt_flac_fr_read_lpc_subframe(fr, order, bits_per_sample, state, error);
       return;
   }
 
-  *error = SPLT_FLAC_ERR_BAD_SUBFRAME_TYPE;
+  splt_e_set_error_data(state, splt_t_get_filename_to_split(state));
+  *error = SPLT_ERROR_INVALID;
 }
 
 static void splt_flac_fr_read_subframe(splt_flac_frame_reader *fr, unsigned bits_per_sample,
-    splt_flac_code *error)
+    splt_state *state, splt_code *error)
 {
   unsigned char bit_padding_subframe_type_and_wasted_bit = splt_flac_u_read_next_byte(fr, error);
   if (*error < 0) { return; }
@@ -315,7 +321,8 @@ static void splt_flac_fr_read_subframe(splt_flac_frame_reader *fr, unsigned bits
   unsigned char zero_bit = bit_padding_subframe_type_and_wasted_bit >> 7;
   if (zero_bit != 0)
   {
-    *error = SPLT_FLAC_ERR_ZERO_BIT_OF_SUBFRAME_HEADER_IS_WRONG;
+    splt_e_set_error_data(state, splt_t_get_filename_to_split(state));
+    *error = SPLT_ERROR_INVALID;
     return;
   }
 
@@ -341,7 +348,8 @@ static void splt_flac_fr_read_subframe(splt_flac_frame_reader *fr, unsigned bits
 
   if (subframe_type & 0x80)
   {
-    *error = SPLT_FLAC_ERR_LOST_SYNC;
+    splt_e_set_error_data(state, splt_t_get_filename_to_split(state));
+    *error = SPLT_ERROR_INVALID;
     return;
   }
 
@@ -355,7 +363,8 @@ static void splt_flac_fr_read_subframe(splt_flac_frame_reader *fr, unsigned bits
   }
   else if (subframe_type < 16)
   {
-    *error = SPLT_FLAC_ERR_LOST_SYNC;
+    splt_e_set_error_data(state, splt_t_get_filename_to_split(state));
+    *error = SPLT_ERROR_INVALID;
     return;
   }
   else if (subframe_type <= 24)
@@ -365,7 +374,8 @@ static void splt_flac_fr_read_subframe(splt_flac_frame_reader *fr, unsigned bits
   }
   else if (subframe_type < 64)
   {
-    *error = SPLT_FLAC_ERR_LOST_SYNC;
+    splt_e_set_error_data(state, splt_t_get_filename_to_split(state));
+    *error = SPLT_ERROR_INVALID;
     return;
   }
   else
@@ -374,16 +384,17 @@ static void splt_flac_fr_read_subframe(splt_flac_frame_reader *fr, unsigned bits
     order = ((bit_padding_subframe_type_and_wasted_bit >> 1) & 31) + 1;
   }
 
-  splt_flac_fr_read_subframe_content(fr, sf_type, order, bits_per_sample - wasted_bits_per_sample, error);
+  splt_flac_fr_read_subframe_content(fr, sf_type, order, bits_per_sample - wasted_bits_per_sample,
+      state, error);
 }
 
 static void splt_flac_fr_read_frame(splt_flac_frame_reader *fr,
     unsigned min_blocksize, unsigned max_blocksize, unsigned metadata_bits_per_sample,
-    splt_flac_code *error)
+    splt_state *state, splt_code *error)
 {
   fr->crc16 = 0;
 
-  splt_flac_fr_read_header(fr, min_blocksize, max_blocksize, metadata_bits_per_sample, error);
+  splt_flac_fr_read_header(fr, min_blocksize, max_blocksize, metadata_bits_per_sample, state, error);
   if (*error < 0) { return; }
 
   unsigned channel = 0;
@@ -395,7 +406,7 @@ static void splt_flac_fr_read_frame(splt_flac_frame_reader *fr,
         (fr->channel_assignment == SPLT_FLAC_MID_SIDE && channel == 1))
       bits_per_sample++;
 
-    splt_flac_fr_read_subframe(fr, bits_per_sample, error);
+    splt_flac_fr_read_subframe(fr, bits_per_sample, state, error);
     if (*error < 0) { return; }
   }
 
@@ -406,26 +417,14 @@ static void splt_flac_fr_read_frame(splt_flac_frame_reader *fr,
   unsigned crc16 = splt_flac_u_read_unsigned(fr, error);
   if (*error < 0) { return; }
 
-  if (crc16 != computed_crc16) { *error = SPLT_FLAC_ERR_BAD_CRC16; }
-}
-
-static splt_code splt_flac_fr_convert_flac_error_to_splt_error(splt_state *state, int frame_reader_error_code)
-{
-  if (frame_reader_error_code >= -1)
+  if (crc16 != computed_crc16)
   {
-    //TODO: EOF
-    return SPLT_OK;
+    splt_e_set_error_data(state, splt_t_get_filename_to_split(state));
+    *error = SPLT_ERROR_INVALID;
   }
-
-  splt_d_print_debug(state, "flac error code %d\n", frame_reader_error_code);
-
-  //TODO: switch case ...
-
-  return SPLT_ERROR_INVALID;
-  //TODO: set filename ?
 }
 
-static void splt_flac_fr_set_next_frame_and_sample_numbers(splt_flac_frame_reader *fr, int *error)
+static void splt_flac_fr_set_next_frame_and_sample_numbers(splt_flac_frame_reader *fr, splt_code *error)
 {
   fr->frame_number++;
   fr->sample_number = fr->sample_number + fr->blocksize;
@@ -433,16 +432,16 @@ static void splt_flac_fr_set_next_frame_and_sample_numbers(splt_flac_frame_reade
   if (fr->frame_number_as_utf8) { free(fr->frame_number_as_utf8); }
   fr->frame_number_as_utf8 =
     splt_flac_l_convert_to_utf8((FLAC__uint64) fr->frame_number, &fr->frame_number_as_utf8_length);
-  if (fr->frame_number_as_utf8 == NULL) { *error = SPLT_FLAC_ERR_CANNOT_ALLOCATE_MEMORY; return; }
+  if (fr->frame_number_as_utf8 == NULL) { *error = SPLT_ERROR_CANNOT_ALLOCATE_MEMORY; return; }
 
   if (fr->sample_number_as_utf8) { free(fr->sample_number_as_utf8); }
   fr->sample_number_as_utf8 =
     splt_flac_l_convert_to_utf8((FLAC__uint64) fr->sample_number, &fr->sample_number_as_utf8_length);
-  if (fr->sample_number_as_utf8 == NULL) { *error = SPLT_FLAC_ERR_CANNOT_ALLOCATE_MEMORY; return; }
+  if (fr->sample_number_as_utf8 == NULL) { *error = SPLT_ERROR_CANNOT_ALLOCATE_MEMORY; return; }
 }
 
 static void splt_flac_fr_write_frame_processor(unsigned char *frame, size_t frame_length,
-    splt_flac_code *error, void *user_data)
+    splt_state *state, splt_code *error, void *user_data)
 {
   splt_flac_frame_reader *fr = (splt_flac_frame_reader *) user_data;
 
@@ -482,7 +481,7 @@ static void splt_flac_fr_write_frame_processor(unsigned char *frame, size_t fram
   unsigned char *modified_frame = malloc(sizeof(unsigned char) * modified_frame_length);
   if (modified_frame == NULL)
   {
-    *error = SPLT_FLAC_ERR_CANNOT_ALLOCATE_MEMORY;
+    *error = SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
     return;
   }
 
@@ -533,17 +532,17 @@ static void splt_flac_fr_write_frame_processor(unsigned char *frame, size_t fram
   modified_frame[j] = first_byte_of_new_crc16;
   modified_frame[j+1] = last_byte_of_new_crc16;
 
-  //TODO: change with splt_io_fwrite
-  if (fwrite(modified_frame, modified_frame_length, 1, fr->out) != 1)
+  if (splt_io_fwrite(state, modified_frame, modified_frame_length, 1, fr->out) != 1)
   {
-    *error = SPLT_FLAC_ERR_FAILED_TO_WRITE_OUTPUT_FILE;
+    splt_e_set_error_data(state, fr->output_fname);
+    *error = SPLT_ERROR_CANT_WRITE_TO_OUTPUT_FILE;
   }
 
   free(modified_frame);
 }
 
 static void splt_flac_fr_backup_frame_processor(unsigned char *frame, size_t frame_length,
-    splt_flac_code *error, void *user_data)
+    splt_state *state, splt_code *error, void *user_data)
 {
   splt_flac_frame_reader *fr = (splt_flac_frame_reader *) user_data;
 
@@ -556,7 +555,7 @@ static void splt_flac_fr_backup_frame_processor(unsigned char *frame, size_t fra
 static void splt_flac_fr_finish_and_write_streaminfo(splt_state *state,
     unsigned min_blocksize, unsigned max_blocksize,
     unsigned min_framesize, unsigned max_framesize,
-    splt_flac_frame_reader *fr, splt_flac_code *error)
+    const splt_flac_metadatas *metadatas, splt_flac_frame_reader *fr, splt_code *error)
 {
   if (min_framesize == 0) { fr->out_streaminfo.min_framesize = 0; }
   if (max_framesize == 0) { fr->out_streaminfo.max_framesize = 0; }
@@ -570,16 +569,19 @@ static void splt_flac_fr_finish_and_write_streaminfo(splt_state *state,
   unsigned char *streaminfo_bytes = splt_flac_l_convert_from_streaminfo(&fr->out_streaminfo);
   if (streaminfo_bytes == NULL)
   {
-    *error = SPLT_FLAC_ERR_CANNOT_ALLOCATE_MEMORY;
+    *error = SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
     return;
   }
 
+  int last = 1;
+  if (metadatas->number_of_datas > 0) { last = 0; }
+
   unsigned char *metadata_header =
-    splt_flac_mu_build_metadata_header(SPLT_FLAC_METADATA_STREAMINFO, 1, SPLT_FLAC_STREAMINFO_LENGTH);
+    splt_flac_mu_build_metadata_header(SPLT_FLAC_METADATA_STREAMINFO, last, SPLT_FLAC_STREAMINFO_LENGTH);
   if (metadata_header == NULL)
   {
     free(streaminfo_bytes);
-    *error = SPLT_FLAC_ERR_CANNOT_ALLOCATE_MEMORY;
+    *error = SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
     return;
   }
 
@@ -587,25 +589,24 @@ static void splt_flac_fr_finish_and_write_streaminfo(splt_state *state,
   rewind(fr->out);
 
   unsigned char flac_word[4] = { 0x66, 0x4C, 0x61, 0x43 };
-  //TODO: change with splt_io_fwrite
-  if (fwrite(flac_word, 4, 1, fr->out) != 1)
+  if (splt_io_fwrite(state, flac_word, 4, 1, fr->out) != 1)
   {
-    *error = SPLT_FLAC_ERR_FAILED_TO_WRITE_OUTPUT_FILE;
+    splt_e_set_error_data(state, fr->output_fname);
+    *error = SPLT_ERROR_CANT_WRITE_TO_OUTPUT_FILE;
     goto end;
   }
 
-  //TODO: change is_last from 1 to 0 when writing other metadata types ...
-  //TODO: change with splt_io_fwrite
-  if (fwrite(metadata_header, SPLT_FLAC_METADATA_HEADER_LENGTH, 1, fr->out) != 1)
+  if (splt_io_fwrite(state, metadata_header, SPLT_FLAC_METADATA_HEADER_LENGTH, 1, fr->out) != 1)
   {
-    *error = SPLT_FLAC_ERR_FAILED_TO_WRITE_OUTPUT_FILE;
+    splt_e_set_error_data(state, fr->output_fname);
+    *error = SPLT_ERROR_CANT_WRITE_TO_OUTPUT_FILE;
     goto end;
   }
 
-  //TODO: change with splt_io_fwrite
-  if (fwrite(streaminfo_bytes, SPLT_FLAC_STREAMINFO_LENGTH, 1, fr->out) != 1)
+  if (splt_io_fwrite(state, streaminfo_bytes, SPLT_FLAC_STREAMINFO_LENGTH, 1, fr->out) != 1)
   {
-    *error = SPLT_FLAC_ERR_FAILED_TO_WRITE_OUTPUT_FILE;
+    splt_e_set_error_data(state, fr->output_fname);
+    *error = SPLT_ERROR_CANT_WRITE_TO_OUTPUT_FILE;
   }
 
 end:
@@ -650,10 +651,13 @@ static splt_flac_frame_reader *splt_flac_fr_reset_for_new_file(splt_flac_frame_r
     return NULL;
   }
 
+  if (fr->output_fname) { free(fr->output_fname); }
+  fr->output_fname = NULL;
+
   return fr;
 }
 
-splt_flac_frame_reader *splt_flac_fr_new(FILE *in)
+splt_flac_frame_reader *splt_flac_fr_new(FILE *in, const char *input_filename)
 {
   splt_flac_frame_reader *fr = malloc(sizeof(splt_flac_frame_reader));
   if (fr == NULL) { return NULL; }
@@ -689,6 +693,8 @@ splt_flac_frame_reader *splt_flac_fr_new(FILE *in)
   fr->previous_frame = NULL;
   fr->previous_frame_length = 0;
 
+  fr->end_point = 0;
+
   return fr;
 }
 
@@ -701,47 +707,236 @@ void splt_flac_fr_free(splt_flac_frame_reader *fr)
   if (fr->buffer) { free(fr->buffer); }
   if (fr->output_buffer) { free(fr->output_buffer); }
   if (fr->previous_frame) { free(fr->previous_frame); }
+  if (fr->output_fname) { free(fr->output_fname); }
 
   free(fr);
 }
 
-static void splt_flac_fr_open_file_and_reserve_streaminfo_space_if_first_time(splt_flac_frame_reader *fr,
-    const char *output_fname, splt_flac_code *error)
+static void splt_flac_fr_write_metadatas(splt_flac_frame_reader *fr,
+    const splt_flac_metadatas *metadatas, const int set_last_as_last,
+    const char *output_fname, splt_state *state, splt_code *error)
+{
+  int i = 0;
+  for (;i < metadatas->number_of_datas; i++)
+  {
+    splt_flac_one_metadata *one_metadata = &metadatas->datas[i];
+
+    unsigned char block_flag_and_block_type = one_metadata->block_type;
+    if (set_last_as_last && (i == metadatas->number_of_datas - 1))
+    {
+      block_flag_and_block_type |= 0x80;
+    }
+
+    FLAC__uint32 block_length = one_metadata->block_length;
+    FLAC__byte block_lengths[3];
+    splt_flac_l_pack_uint32(block_length, block_lengths, 3);
+
+    if (splt_io_fwrite(state, &block_flag_and_block_type, 1, 1, fr->out) != 1) { goto error; }
+    if (splt_io_fwrite(state, block_lengths, 3, 1, fr->out) != 1) { goto error; }
+    if (splt_io_fwrite(state, one_metadata->bytes, block_length, 1, fr->out) != 1) { goto error; }
+  }
+
+  return;
+
+error:
+  splt_e_set_error_data(state, fr->output_fname);
+  *error = SPLT_ERROR_CANT_WRITE_TO_OUTPUT_FILE;
+}
+
+static void splt_flac_fr_write_tags(splt_flac_frame_reader *fr, const splt_flac_tags *flac_tags,
+    const splt_tags *tags, const char *output_fname, splt_state *state, splt_code *error)
+{
+  splt_flac_vorbis_tags *vorbis_tags = splt_flac_vorbis_tags_new(error);
+  if (*error < 0) { return; }
+
+  splt_flac_vorbis_tags_append_with_prefix(vorbis_tags, "TITLE=", tags->title, error);
+  if (*error < 0) { goto end; }
+  char *artist_or_performer = splt_tu_get_artist_or_performer_ptr(tags);
+  splt_flac_vorbis_tags_append_with_prefix(vorbis_tags, "ARTIST=", artist_or_performer, error);
+  if (*error < 0) { goto end; }
+  splt_flac_vorbis_tags_append_with_prefix(vorbis_tags, "ALBUM=", tags->album, error);
+  if (*error < 0) { goto end; }
+  splt_flac_vorbis_tags_append_with_prefix(vorbis_tags, "DATE=", tags->year, error);
+  if (*error < 0) { goto end; }
+  splt_flac_vorbis_tags_append_with_prefix(vorbis_tags, "GENRE=", tags->genre, error);
+  if (*error < 0) { goto end; }
+  splt_flac_vorbis_tags_append_with_prefix(vorbis_tags, "COMMENT=", tags->comment, error);
+  if (*error < 0) { goto end; }
+  char track_str[255] = { '\0' };
+  snprintf(track_str, 254, "%d", tags->track);
+  splt_flac_vorbis_tags_append_with_prefix(vorbis_tags, "TRACKNUMBER=", track_str, error);
+  if (*error < 0) { goto end; }
+
+  if (tags->set_original_tags && flac_tags->other_tags)
+  {
+    FLAC__uint32 j = 0;
+    for (;j < flac_tags->other_tags->number_of_tags; j++)
+    {
+      splt_flac_vorbis_tags_append(vorbis_tags, flac_tags->other_tags->tags[j], error);
+      if (*error < 0) { goto end; }
+    }
+  }
+
+  //4 is VORBIS_COMMENT metadata block type
+  unsigned char block_flag_and_block_type = 4;
+  //set as true the last metadata block flag
+  block_flag_and_block_type |= 0x80;
+
+  FLAC__uint32 total_bytes = 4 + flac_tags->vendor_length + 4 + vorbis_tags->total_bytes;
+
+  FLAC__byte block_length[3];
+  splt_flac_l_pack_uint32(total_bytes, block_length, 3);
+
+  //metadata block header
+  if (splt_io_fwrite(state, &block_flag_and_block_type, 1, 1, fr->out) != 1) { goto error; }
+  if (splt_io_fwrite(state, block_length, 3, 1, fr->out) != 1) { goto error; }
+
+  //metadata vorbis block
+  FLAC__byte uint32_bytes[4];
+
+  //vendor length and string
+  splt_flac_l_pack_uint32_little_endian(flac_tags->vendor_length, uint32_bytes, 4);
+  if (splt_io_fwrite(state, uint32_bytes, 4, 1, fr->out) != 1) { goto error; }
+  if (splt_io_fwrite(state, flac_tags->vendor_string, flac_tags->vendor_length, 1, fr->out) != 1)
+  {
+    goto error;
+  }
+
+  //number of comments
+  splt_flac_l_pack_uint32_little_endian(vorbis_tags->number_of_tags, uint32_bytes, 4);
+  if (splt_io_fwrite(state, uint32_bytes, 4, 1, fr->out) != 1) { goto error; }
+
+  FLAC__uint32 i = 0;
+  for (;i < vorbis_tags->number_of_tags;i++)
+  {
+    FLAC__uint32 comment_length = (FLAC__uint32) strlen(vorbis_tags->tags[i]);
+    splt_flac_l_pack_uint32_little_endian(comment_length, uint32_bytes, 4);
+    //comment length and value
+    if (splt_io_fwrite(state, uint32_bytes, 4, 1, fr->out) != 1) { goto error; }
+    if (splt_io_fwrite(state, vorbis_tags->tags[i], comment_length, 1, fr->out) != 1) { goto error; }
+  }
+
+end:
+  splt_flac_vorbis_tags_free(&vorbis_tags); 
+  return;
+
+error:
+  splt_e_set_error_data(state, fr->output_fname);
+  *error = SPLT_ERROR_CANT_WRITE_TO_OUTPUT_FILE;
+  splt_flac_vorbis_tags_free(&vorbis_tags); 
+}
+
+static void splt_flac_fr_open_file_and_write_metadata_if_first_time(splt_flac_frame_reader *fr,
+    const splt_flac_metadatas *metadatas, const splt_flac_tags *flac_tags, 
+    const splt_tags *tags_to_write, const char *output_fname, splt_state *state, splt_code *error)
 {
   if (fr->out_streaminfo.total_samples != 0)
   {
     return;
   }
 
+  splt_c_put_progress_text(state, SPLT_PROGRESS_CREATE);
+
   fr->out = splt_io_fopen(output_fname, "wb+");
   if (fr->out == NULL)
   {
-    *error = SPLT_FLAC_ERR_FAILED_TO_OPEN_FILE;
+    splt_e_set_strerror_msg_with_data(state, output_fname);
+    *error = SPLT_ERROR_CANNOT_OPEN_DEST_FILE;
     return;
   }
 
-  unsigned char space[4+SPLT_FLAC_METADATA_HEADER_LENGTH+SPLT_FLAC_STREAMINFO_LENGTH]={'\0'};
+  unsigned char space[4+SPLT_FLAC_METADATA_HEADER_LENGTH+SPLT_FLAC_STREAMINFO_LENGTH] = {'\0'};
   int space_size= 4 + SPLT_FLAC_METADATA_HEADER_LENGTH + SPLT_FLAC_STREAMINFO_LENGTH;
-  //TODO: change with splt_io_fwrite
-  if (fwrite(space, space_size, 1, fr->out) != 1)
+  if (splt_io_fwrite(state, space, space_size, 1, fr->out) != 1)
   {
-    *error = SPLT_FLAC_ERR_FAILED_TO_WRITE_OUTPUT_FILE;
+    splt_e_set_error_data(state, fr->output_fname);
+    *error = SPLT_ERROR_CANT_WRITE_TO_OUTPUT_FILE;
+  }
+
+  int need_to_write_tags = SPLT_TRUE;
+  if (flac_tags == NULL || tags_to_write == NULL)
+  {
+    need_to_write_tags = SPLT_FALSE;
+  }
+
+  splt_flac_fr_write_metadatas(fr, metadatas, !need_to_write_tags, output_fname, state, error);
+  if (*error < 0) { return; }
+
+  if (need_to_write_tags)
+  {
+    splt_flac_fr_write_tags(fr, flac_tags, tags_to_write, output_fname, state, error);
   }
 }
 
-void splt_flac_fr_read_and_write_frames(splt_state *state, splt_flac_frame_reader *fr, 
+static double splt_flac_fr_back_end_point_according_to_auto_adjust(splt_state *state, 
+    double begin_point, double end_point, int *adjust_gap_secs)
+{
+  if (!*adjust_gap_secs)
+  {
+    return end_point;
+  }
+
+  double new_end_point = end_point;
+
+  double adj = (double) (*adjust_gap_secs);
+  double len = (end_point - begin_point);
+  if (adj > len)
+  {
+    adj = len;
+  }
+
+  if (end_point > adj)
+  {
+    new_end_point -= adj;
+    *adjust_gap_secs = (int) adj;
+  }
+  else
+  {
+    *adjust_gap_secs = 0;
+  }
+
+  return new_end_point;
+}
+
+static void update_progress(splt_state *state, double first_time, double time, double end_point,
+    short before_adjust)
+{
+  double current = time - first_time;
+  double total = end_point - first_time;
+
+  if ((splt_o_get_int_option(state, SPLT_OPT_SPLIT_MODE) == SPLT_OPTION_SILENCE_MODE) ||
+      (splt_o_get_int_option(state, SPLT_OPT_SPLIT_MODE) == SPLT_OPTION_TRIM_SILENCE_MODE) ||
+      (!splt_o_get_int_option(state, SPLT_OPT_AUTO_ADJUST)))
+  {
+    splt_c_update_progress(state, current, total, 1, 0, SPLT_DEFAULT_PROGRESS_RATE2);
+  }
+  else
+  {
+    if (before_adjust)
+    {
+      splt_c_update_progress(state, current, total, 2, 0, SPLT_DEFAULT_PROGRESS_RATE2);
+    }
+    else
+    {
+      splt_c_update_progress(state, current, total, 4, 0.75, SPLT_DEFAULT_PROGRESS_RATE2);
+    }
+  }
+}
+
+void splt_flac_fr_read_and_write_frames(splt_state *state, splt_flac_frame_reader *fr,
+    const splt_flac_metadatas *metadatas, const splt_flac_tags *flac_tags,
+    const splt_tags *tags_to_write,
     const char *output_fname,
     double begin_point, double end_point, int save_end_point,
     unsigned min_blocksize, unsigned max_blocksize, 
     unsigned bits_per_sample, unsigned sample_rate, unsigned channels, 
     unsigned min_framesize, unsigned max_framesize,
-    int *error)
+    float offset,
+    splt_code *error)
 {
-  int frame_reader_error_code = SPLT_FLAC_OK;
-
   if (splt_flac_fr_reset_for_new_file(fr) == NULL)
   {
-    frame_reader_error_code = SPLT_FLAC_ERR_CANNOT_ALLOCATE_MEMORY;
+    *error = SPLT_ERROR_CANNOT_ALLOCATE_MEMORY;
     goto end;
   }
 
@@ -749,73 +944,155 @@ void splt_flac_fr_read_and_write_frames(splt_state *state, splt_flac_frame_reade
   fr->out_streaminfo.channels = channels;
   fr->out_streaminfo.bits_per_sample = bits_per_sample;
 
+  splt_su_copy(output_fname, &fr->output_fname);
+
   if (save_end_point && fr->previous_frame)
   {
-    splt_flac_fr_open_file_and_reserve_streaminfo_space_if_first_time(fr, output_fname, 
-        &frame_reader_error_code);
+    splt_flac_fr_open_file_and_write_metadata_if_first_time(fr, metadatas, flac_tags,
+        tags_to_write, output_fname, state, error);
 
-    splt_flac_fr_write_frame_processor(fr->previous_frame, fr->previous_frame_length,
-        &frame_reader_error_code, fr);
+    splt_flac_fr_write_frame_processor(fr->previous_frame, fr->previous_frame_length, state, error, fr);
     free(fr->previous_frame);
     fr->previous_frame = NULL;
     fr->previous_frame_length = 0;
 
-    splt_flac_fr_set_next_frame_and_sample_numbers(fr, &frame_reader_error_code);
-    if (frame_reader_error_code < 0) { goto end; }
+    splt_flac_fr_set_next_frame_and_sample_numbers(fr, error);
+    if (*error < 0) { goto end; }
 
     fr->out_streaminfo.total_samples += fr->blocksize;
   }
 
+  if (save_end_point && fr->end_point > 0)
+  {
+    begin_point = fr->end_point;
+  }
+
+  unsigned long adjust_gap_hundr = 0;
+  int adjust_gap_secs = splt_o_get_int_option(state, SPLT_OPT_PARAM_GAP);
+
+  short end_point_is_eof = splt_u_fend_sec_is_bigger_than_total_time(state, end_point);
+  double total_time = splt_t_get_total_time_as_double_secs(state);
+  if (!end_point_is_eof && end_point < total_time)
+  {
+    end_point = splt_flac_fr_back_end_point_according_to_auto_adjust(state,
+        begin_point, end_point, &adjust_gap_secs);
+    adjust_gap_hundr = (unsigned long) adjust_gap_secs * 100;
+  }
+
+  off_t previous_offset = 0;
+
   int we_continue = 1;
+  double first_time = -1;
+  short before_adjust = SPLT_TRUE;
+
   while (we_continue)
   {
     unsigned frame_byte_buffer_start = 0;
     if (fr->buffer != NULL) { frame_byte_buffer_start = fr->next_byte; }
 
-    splt_flac_fr_read_frame(fr, min_blocksize, max_blocksize, bits_per_sample, 
-        &frame_reader_error_code);
-    if (frame_reader_error_code < 0) { goto end; }
+    splt_flac_fr_read_frame(fr, min_blocksize, max_blocksize, bits_per_sample, state, error);
+    if (*error < 0) { goto end; }
 
     double time = (double) fr->current_sample_number / (double) sample_rate;
+    if (first_time < 0) { first_time = time; }
+
+    update_progress(state, first_time, time, end_point, before_adjust);
+
     if (time >= begin_point && (time < end_point || end_point < 0))
     {
-      splt_flac_fr_open_file_and_reserve_streaminfo_space_if_first_time(fr, output_fname, 
-          &frame_reader_error_code);
+      splt_flac_fr_open_file_and_write_metadata_if_first_time(fr, metadatas, flac_tags,
+          tags_to_write, output_fname, state, error);
 
-      splt_flac_u_process_frame(fr, frame_byte_buffer_start, &frame_reader_error_code,
+      splt_flac_u_process_frame(fr, frame_byte_buffer_start, state, error,
           splt_flac_fr_write_frame_processor, fr);
-      if (frame_reader_error_code < 0) { goto end; }
+      if (*error < 0) { goto end; }
 
-      splt_flac_fr_set_next_frame_and_sample_numbers(fr, &frame_reader_error_code);
-      if (frame_reader_error_code < 0) { goto end; }
+      splt_flac_fr_set_next_frame_and_sample_numbers(fr, error);
+      if (*error < 0) { goto end; }
 
       fr->out_streaminfo.total_samples += fr->blocksize;
     }
     else if (end_point > 0 && time >= end_point)
     {
-      splt_flac_u_process_frame(fr, frame_byte_buffer_start, &frame_reader_error_code, 
-          splt_flac_fr_backup_frame_processor, fr);
-      we_continue = 0;
+      if (adjust_gap_hundr)
+      {
+        float threshold = splt_o_get_float_option(state, SPLT_OPT_PARAM_THRESHOLD);
+        float min_length = splt_o_get_float_option(state, SPLT_OPT_PARAM_MIN_LENGTH);
+        int shots = splt_o_get_int_option(state, SPLT_OPT_PARAM_SHOTS);
+
+        unsigned long length = 2 * adjust_gap_hundr;
+
+        int silence_points_found = splt_flac_scan_silence(state, previous_offset, length,
+            threshold, min_length, shots, 0, error, splt_scan_silence_processor);
+
+        if (silence_points_found > 0)
+        {
+          end_point = (double) splt_siu_silence_position(state->silence_list, offset);
+        }
+        else
+        {
+          end_point += (double) adjust_gap_secs;
+        }
+
+        splt_siu_ssplit_free(&state->silence_list);
+        adjust_gap_hundr = 0;
+        splt_c_put_progress_text(state, SPLT_PROGRESS_CREATE);
+
+        before_adjust = SPLT_FALSE;
+        first_time = time;
+      }
+
+      if (end_point > 0 && time >= end_point)
+      {
+        splt_flac_u_process_frame(fr, frame_byte_buffer_start, state, error,
+            splt_flac_fr_backup_frame_processor, fr);
+        we_continue = 0;
+      }
+      else
+      {
+        //process frame if auto adjust changed the end point
+        splt_flac_u_process_frame(fr, frame_byte_buffer_start, state, error,
+            splt_flac_fr_write_frame_processor, fr);
+        if (*error < 0) { goto end; }
+
+        splt_flac_fr_set_next_frame_and_sample_numbers(fr, error);
+        if (*error < 0) { goto end; }
+
+        fr->out_streaminfo.total_samples += fr->blocksize;
+      }
     }
     else
     {
-      splt_flac_u_process_frame(fr, frame_byte_buffer_start, &frame_reader_error_code, NULL, fr);
+      splt_c_put_progress_text(state, SPLT_PROGRESS_PREPARE);
+      splt_flac_u_process_frame(fr, frame_byte_buffer_start, state, error, NULL, fr);
     }
 
     if (feof(fr->in) && fr->next_byte >= fr->read_bytes)
     {
+      *error = SPLT_OK_SPLIT_EOF;
       break;
     }
+
+    previous_offset = ftello(fr->in) - (fr->read_bytes - fr->next_byte);
   }
 
   if (fr->out_streaminfo.total_samples != 0)
   {
     splt_flac_fr_finish_and_write_streaminfo(state, min_blocksize, max_blocksize,
-        min_framesize, max_framesize, fr, &frame_reader_error_code);
+        min_framesize, max_framesize, metadatas, fr, error);
   }
   else
   {
-    frame_reader_error_code = SPLT_FLAC_ERR_BEGIN_OUT_OF_FILE;
+    *error = SPLT_ERROR_BEGIN_OUT_OF_FILE;
+  }
+
+  if (save_end_point)
+  {
+    fr->end_point = end_point;
+  }
+  else
+  {
+    fr->end_point = 0;
   }
 
 end:
@@ -830,8 +1107,5 @@ end:
       return;
     }
   }
-
-  splt_code err = splt_flac_fr_convert_flac_error_to_splt_error(state, frame_reader_error_code);
-  if (err < 0) { *error = err; }
 }
 
