@@ -3,7 +3,7 @@
  *               for mp3/ogg splitting without decoding
  *
  * Copyright (c) 2002-2005 M. Trotta - <mtrotta@users.sourceforge.net>
- * Copyright (c) 2005-2012 Alexandru Munteanu - io_fx@yahoo.fr
+ * Copyright (c) 2005-2013 Alexandru Munteanu - m@ioalex.net
  *
  *********************************************************/
 
@@ -20,8 +20,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307,
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
  * USA.
  *********************************************************/
 
@@ -44,12 +43,13 @@ typedef struct {
   short set_new_length;
 
   short continue_after_silence;
+  double previous_time;
 } splt_scan_silence_data;
 
-short splt_scan_silence_processor(double time, int silence_was_found, short must_flush, 
+short splt_scan_silence_processor(double time, float level, int silence_was_found, short must_flush, 
     splt_scan_silence_data *data, int *found_silence_points, int *error);
 
-short splt_trim_silence_processor(double time, int silence_was_found, short must_flush,
+short splt_trim_silence_processor(double time, float level, int silence_was_found, short must_flush,
     splt_scan_silence_data *data, int *found_silence_points, int *error);
 
 splt_scan_silence_data *splt_scan_silence_data_new(splt_state *state, short first, 
