@@ -4,7 +4,7 @@
  *               for mp3/ogg splitting without decoding
  *
  * Copyright (c) 2002-2005 M. Trotta - <mtrotta@users.sourceforge.net>
- * Copyright (c) 2005-2012 Munteanu Alexandru - io_fx@yahoo.fr
+ * Copyright (c) 2005-2013 Alexandru Munteanu - m@ioalex.net
  *
  * http://mp3splt.sourceforge.net
  *
@@ -24,8 +24,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307,
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
  * USA.
  *
  *********************************************************/
@@ -33,8 +32,8 @@
 #ifndef SPLT_CONVERSIONS_H
 
 long splt_co_convert_to_hundreths (const char *s);
-float splt_co_convert_to_dB(double input);
-double splt_co_convert_from_dB(float input);
+float splt_co_convert_to_db(double input);
+double splt_co_convert_from_db(float input);
 void splt_co_get_mins_secs_hundr(long split_hundr, long *mins, long *secs, long *hundr);
 long splt_co_time_to_long(double time);
 long splt_co_time_to_long_ceil(double time);
