@@ -4,7 +4,7 @@
  *               for mp3/ogg splitting without decoding
  *
  * Copyright (c) 2002-2005 M. Trotta - <mtrotta@users.sourceforge.net>
- * Copyright (c) 2005-2012 Alexandru Munteanu - io_fx@yahoo.fr
+ * Copyright (c) 2005-2013 Alexandru Munteanu - m@ioalex.net
  *
  * http://mp3splt.sourceforge.net
  *
@@ -24,8 +24,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307,
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
  * USA.
  *
  *********************************************************/
@@ -36,9 +35,9 @@
 */
 #include "splt.h"
 
-splt_pair *splt_pair_new(void *first, void *second)
+splt_int_pair *splt_int_pair_new(int first, int second)
 {
-  splt_pair *pair = malloc(sizeof(splt_pair));
+  splt_int_pair *pair = malloc(sizeof(splt_int_pair));
   if (pair == NULL)
   {
     return NULL;
@@ -50,7 +49,7 @@ splt_pair *splt_pair_new(void *first, void *second)
   return pair;
 }
 
-void splt_pair_free(splt_pair **pair)
+void splt_int_pair_free(splt_int_pair **pair)
 {
   if (!pair)
   {
@@ -66,12 +65,52 @@ void splt_pair_free(splt_pair **pair)
   *pair = NULL;
 }
 
-void *splt_pair_first(splt_pair *pair)
+int splt_int_pair_first(splt_int_pair *pair)
 {
   return pair->first;
 }
 
-void *splt_pair_second(splt_pair *pair)
+int splt_int_pair_second(splt_int_pair *pair)
+{
+  return pair->second;
+}
+
+splt_il_pair *splt_il_pair_new(int first, long second)
+{
+  splt_il_pair *pair = malloc(sizeof(splt_il_pair));
+  if (pair == NULL)
+  {
+    return NULL;
+  }
+
+  pair->first = first;
+  pair->second = second;
+
+  return pair;
+}
+
+void splt_il_pair_free(splt_il_pair **pair)
+{
+  if (!pair)
+  {
+    return;
+  }
+
+  if (!*pair)
+  {
+    return;
+  }
+
+  free(*pair);
+  *pair = NULL;
+}
+
+int splt_il_pair_first(splt_il_pair *pair)
+{
+  return pair->first;
+}
+
+long splt_il_pair_second(splt_il_pair *pair)
 {
   return pair->second;
 }
