@@ -4,7 +4,7 @@
  *               for mp3/ogg splitting without decoding
  *
  * Copyright (c) 2002-2005 M. Trotta - <mtrotta@users.sourceforge.net>
- * Copyright (c) 2005-2012 Munteanu Alexandru - io_fx@yahoo.fr
+ * Copyright (c) 2005-2013 Alexandru Munteanu - m@ioalex.net
  *
  * http://mp3splt.sourceforge.net
  *
@@ -24,8 +24,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307,
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
  * USA.
  *
  *********************************************************/
@@ -33,17 +32,24 @@
 #ifndef SPLT_PAIR_H
 
 typedef struct {
-  void *first;
-  void *second;
-} splt_pair;
+  int first;
+  int second;
+} splt_int_pair;
 
-splt_pair *splt_pair_new(void *first, void *second);
+splt_int_pair *splt_int_pair_new(int first, int second);
+void splt_int_pair_free(splt_int_pair **pair);
+int splt_int_pair_first(splt_int_pair *pair);
+int splt_int_pair_second(splt_int_pair *pair);
 
-void splt_pair_free(splt_pair **pair);
+typedef struct {
+  int first;
+  long second;
+} splt_il_pair;
 
-void *splt_pair_first(splt_pair *pair);
-
-void *splt_pair_second(splt_pair *pair);
+splt_il_pair *splt_il_pair_new(int first, long second);
+void splt_il_pair_free(splt_il_pair **pair);
+int splt_il_pair_first(splt_il_pair *pair);
+long splt_il_pair_second(splt_il_pair *pair);
 
 #define SPLT_PAIR_H
 
