@@ -30,42 +30,13 @@
 #include "splt.h"
 #include "tags_utils.h"
 
+static void splt_tu_copy_tags_without_original_and_version(splt_tags *from, splt_tags *to, 
+    int *error);
+
 void splt_tu_free_original_tags(splt_state *state)
 {
   splt_tags *tags = &state->original_tags.tags;
-
-  if (tags->year)
-  {
-    free(tags->year);
-    tags->year = NULL;
-  }
-  if (tags->artist)
-  {
-    free(tags->artist);
-    tags->artist = NULL;
-  }
-  if (tags->album)
-  {
-    free(tags->album);
-    tags->album = NULL;
-  }
-  if (tags->title)
-  {
-    free(tags->title);
-    tags->title = NULL;
-  }
-  if (tags->comment)
-  {
-    free(tags->comment);
-    tags->comment = NULL;
-  }
-  if (tags->genre)
-  {
-    free(tags->genre);
-    tags->genre = NULL;
-  }
-  tags->track = -1;
-
+  splt_tu_free_one_tags_content(tags);
   int err = SPLT_OK;
   splt_p_clear_original_tags(state, &err);
 }
@@ -530,6 +501,11 @@ int splt_tu_set_original_tags_field(splt_state *state, int tags_field, const voi
   return splt_tu_set_field_on_tags(&state->original_tags.tags, tags_field, data);
 }
 
+void splt_tu_set_to_original_tags(splt_state *state, splt_tags *tags, int *error)
+{
+  splt_tu_copy_tags_without_original_and_version(tags, &state->original_tags.tags, error);
+}
+
 void splt_tu_set_original_tags_data(splt_state *state, void *data)
 {
   state->original_tags.all_original_tags = data;
@@ -945,7 +921,8 @@ int splt_tu_has_one_tag_set(splt_tags *tags)
   return SPLT_FALSE;
 }
 
-void splt_tu_copy_tags(splt_tags *from, splt_tags *to, int *error)
+static void splt_tu_copy_all_tags(splt_tags *from, splt_tags *to, int *error,
+    int copy_original_and_version)
 {
   if (!from)
   {
@@ -975,14 +952,17 @@ void splt_tu_copy_tags(splt_tags *from, splt_tags *to, int *error)
   err = splt_tu_set_field_on_tags(to, SPLT_TAGS_TRACK, &from->track);
   if (err < 0) { goto error; }
 
-  err = splt_tu_set_field_on_tags(to, SPLT_TAGS_ORIGINAL, &from->set_original_tags);
-  if (err < 0) { goto error; }
-
   err = splt_tu_set_field_on_tags(to, SPLT_TAGS_GENRE, from->genre);
   if (err < 0) { goto error; }
 
-  err = splt_tu_set_field_on_tags(to, SPLT_TAGS_VERSION, &from->tags_version);
-  if (err < 0) { goto error; }
+  if (copy_original_and_version)
+  {
+    err = splt_tu_set_field_on_tags(to, SPLT_TAGS_ORIGINAL, &from->set_original_tags);
+    if (err < 0) { goto error; }
+
+    err = splt_tu_set_field_on_tags(to, SPLT_TAGS_VERSION, &from->tags_version);
+    if (err < 0) { goto error; }
+  }
 
   return;
 
@@ -990,6 +970,16 @@ error:
   *error = err;
 }
 
+static void splt_tu_copy_tags_without_original_and_version(splt_tags *from, splt_tags *to, int *error)
+{
+  splt_tu_copy_all_tags(from, to, error, SPLT_FALSE);
+}
+
+void splt_tu_copy_tags(splt_tags *from, splt_tags *to, int *error)
+{
+  splt_tu_copy_all_tags(from, to, error, SPLT_TRUE);
+}
+
 static splt_tags *splt_tu_get_tags_to_replace_in_tags(splt_state *state)
 {
   int current_tags_number = splt_t_get_current_split_file_number(state) - 1;
@@ -1206,7 +1196,7 @@ splt_tags *splt_tu_get_current_tags(splt_state *state)
   return splt_tu_get_tags_at(state, current_tags_number);
 }
 
-char *splt_tu_get_artist_or_performer_ptr(splt_tags *tags)
+char *splt_tu_get_artist_or_performer_ptr(const splt_tags *tags)
 {
   if (!tags)
   {
